package dev.utils.common;

import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dev.utils.JCLogUtils;

/**
 * detail: 字符串工具类
 * @author Ttt
 */
public final class StringUtils {

    private StringUtils() {
    }

    // 日志 TAG
    private static final String TAG = StringUtils.class.getSimpleName();
    // 换行字符串
    public static final String NEW_LINE_STR = System.getProperty("line.separator");
    // 换行字符串 ( 两行 )
    public static final String NEW_LINE_STR_X2 = NEW_LINE_STR + NEW_LINE_STR;

    // ==========
    // = String =
    // ==========

    /**
     * 判断字符串是否为 null
     * @param str 待校验的字符串
     * @return {@code true} is null, {@code false} not null
     */
    public static boolean isEmpty(final String str) {
        return (str == null || str.length() == 0);
    }

    /**
     * 判断字符串是否为 null
     * @param str    待校验的字符串
     * @param isTrim 是否调用 trim()
     * @return {@code true} is null, {@code false} not null
     */
    public static boolean isEmpty(final String str, final boolean isTrim) {
        if (str != null) {
            return isEmpty(isTrim ? str.trim() : str);
        }
        return false;
    }

    /**
     * 判断多个字符串是否存在为 null 的字符串
     * @param strs 待校验的字符串数组
     * @return {@code true} is null, {@code false} not null
     */
    public static boolean isEmpty(final String... strs) {
        if (strs != null && strs.length != 0) {
            for (int i = 0, len = strs.length; i < len; i++) {
                if (isEmpty(strs[i])) {
                    return true;
                }
            }
            return false;
        }
        // 默认表示属于 null
        return true;
    }

    // =

    /**
     * 判断字符串是否不为 null
     * @param str 待校验的字符串
     * @return {@code true} not null, {@code false} is null
     */
    public static boolean isNotEmpty(final String str) {
        return (str != null && str.length() != 0);
    }

    /**
     * 判断字符串是否不为 null
     * @param str    待校验的字符串
     * @param isTrim 是否调用 trim()
     * @return {@code true} not null, {@code false} is null
     */
    public static boolean isNotEmpty(final String str, final boolean isTrim) {
        return isNotEmpty(isTrim ? str.trim() : str);
    }

    // ============
    // = 判断长度 =
    // ============

    /**
     * 获取字符串长度
     * @param str 待校验的字符串
     * @return 字符串长度, 如果字符串为 null, 则返回 0
     */
    public static int length(final String str) {
        return str == null ? 0 : str.length();
    }

    /**
     * 获取字符串长度
     * @param str           待校验的字符串
     * @param defaultLength 字符串为 null 时, 返回的长度
     * @return 字符串长度, 如果字符串为 null, 则返回 defaultLength
     */
    public static int length(final String str, final int defaultLength) {
        return str != null ? str.length() : defaultLength;
    }

    // =

    /**
     * 获取字符串长度 是否等于期望长度
     * @param str    待校验的字符串
     * @param length 期望长度
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isLength(final String str, final int length) {
        return str != null && str.length() == length;
    }

    // ============
    // = 对比判断 =
    // ============

    /**
     * 判断两个值是否一样
     * @param value1 第一个值
     * @param value2 第二个值
     * @param <T>    泛型
     * @return {@code true} yes, {@code false} no
     */
    public static <T> boolean equals(final T value1, final T value2) {
        return ObjectUtils.equals(value1, value2);
    }

    /**
     * 判断多个字符串是否相等, 只有全相等才返回 true - 对比大小写
     * @param strs 待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isEquals(final String... strs) {
        return isEquals(false, strs);
    }

    /**
     * 判断多个字符串是否相等, 只有全相等才返回 true
     * @param isIgnore 是否忽略大小写
     * @param strs     待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isEquals(final boolean isIgnore, final String... strs) {
        if (strs != null) {
            String last = null;
            // 获取数据长度
            int len = strs.length;
            // 如果最多只有一个数据判断, 则直接跳过
            if (len <= 1) {
                return false;
            }
            // 遍历判断
            for (int i = 0; i < len; i++) {
                // 获取临时变量
                String val = strs[i];
                // 如果等于 null, 则跳过
                if (val == null) {
                    return false;
                }
                if (last != null) {
                    if (isIgnore) {
                        if (!val.equalsIgnoreCase(last)) {
                            return false;
                        }
                    } else {
                        if (!val.equals(last)) {
                            return false;
                        }
                    }
                }
                last = val;
            }
            return true;
        }
        return false;
    }

    /**
     * 判断多个字符串, 只要有一个符合条件则通过
     * @param str  待校验的字符串
     * @param strs 待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isOrEquals(final String str, final String... strs) {
        return isOrEquals(false, str, strs);
    }

    /**
     * 判断多个字符串, 只要有一个符合条件则通过
     * @param isIgnore 是否忽略大小写
     * @param str      待校验的字符串
     * @param strs     待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isOrEquals(final boolean isIgnore, final String str, final String... strs) {
        if (str != null && strs != null && strs.length != 0) {
            // 获取数据长度
            int len = strs.length;
            // 遍历判断
            for (int i = 0; i < len; i++) {
                // 获取临时变量
                String val = strs[i];
                // 如果等于 null, 则跳过
                if (val == null) {
                    continue;
                } else {
                    if (isIgnore) {
                        if (val.equalsIgnoreCase(str)) {
                            return true;
                        }
                    } else {
                        if (val.equals(str)) {
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 统计字符串匹配个数
     * @param str     待匹配字符串
     * @param keyword 匹配 key
     * @return 字符串 key 匹配个数
     */
    public static int countMatches(final String str, final String keyword) {
        if (isEmpty(str) || isEmpty(keyword)) return 0;
        try {
            int count = 0;
            Matcher matcher = Pattern.compile(keyword).matcher(str);
            // find() 对字符串进行匹配, 匹配到的字符串可以在任何位置
            while (matcher.find()) {
                count++;
            }
            return count;
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "countMatches");
        }
        return -1;
    }

    /**
     * 统计字符串匹配个数
     * @param str     待匹配字符串
     * @param keyword 匹配 key
     * @return 字符串 key 匹配个数
     */
    public static int countMatches2(final String str, final String keyword) {
        if (isEmpty(str) || isEmpty(keyword)) return 0;
        try {
            // 获取匹配 key 长度
            int keyLength = keyword.length();
            // =
            int count = 0;
            int index = 0;
            while ((index = str.indexOf(keyword, index)) != -1) {
                index = index + keyLength;
                count++;
            }
            return count;
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "countMatches2");
        }
        return -1;
    }

    /**
     * 判断一堆值中, 是否存在符合该条件的 ( 包含 )
     * @param str  待校验的字符串
     * @param strs 待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isContains(final String str, final String... strs) {
        return isContains(false, str, strs);
    }

    /**
     * 判断一堆值中, 是否存在符合该条件的 ( 包含 )
     * @param isIgnore 是否忽略大小写
     * @param str      待校验的字符串
     * @param strs     待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isContains(final boolean isIgnore, final String str, final String... strs) {
        if (str != null && strs != null && strs.length != 0) {
            String tempString = str;
            // 判断是否需要忽略大小写
            if (isIgnore) {
                tempString = tempString.toLowerCase();
            }
            // 获取内容长度
            int strLength = tempString.length();
            // 遍历判断
            for (int i = 0, len = strs.length; i < len; i++) {
                // 获取参数
                String val = strs[i];
                // 判断是否为 null, 或者长度为 0
                if (!isEmpty(val) && strLength != 0) {
                    if (isIgnore) {
                        // 转换小写
                        String valIgnore = val.toLowerCase();
                        // 判断是否包含
                        if (valIgnore.indexOf(tempString) != -1) {
                            return true;
                        }
                    } else {
                        // 判断是否包含
                        if (val.indexOf(tempString) != -1) {
                            return true;
                        }
                    }
                } else {
                    if (tempString.equals(val)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    /**
     * 判断内容, 是否属于特定字符串开头 - 对比大小写
     * @param str  待校验的字符串
     * @param strs 待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isStartsWith(final String str, final String... strs) {
        return isStartsWith(false, str, strs);
    }

    /**
     * 判断内容, 是否属于特定字符串开头
     * @param isIgnore 是否忽略大小写
     * @param str      待校验的字符串
     * @param strs     待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isStartsWith(final boolean isIgnore, final String str, final String... strs) {
        if (!isEmpty(str) && strs != null && strs.length != 0) {
            String tempString = str;
            // 判断是否需要忽略大小写
            if (isIgnore) {
                tempString = tempString.toLowerCase();
            }
            // 获取数据长度
            int len = strs.length;
            // 遍历判断
            for (int i = 0; i < len; i++) {
                // 获取临时变量
                String val = strs[i];
                // 判断是否为 null, 或者长度为 0
                if (!isEmpty(val)) {
                    if (isIgnore) {
                        // 转换小写
                        String valIgnore = val.toLowerCase();
                        // 判断是否属于 val 开头
                        if (tempString.startsWith(valIgnore)) {
                            return true;
                        }
                    } else {
                        // 判断是否属于 val 开头
                        if (tempString.startsWith(val)) {
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    /**
     * 判断内容, 是否属于特定字符串结尾 - 对比大小写
     * @param str  待校验的字符串
     * @param strs 待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isEndsWith(final String str, final String... strs) {
        return isEndsWith(false, str, strs);
    }

    /**
     * 判断内容, 是否属于特定字符串结尾
     * @param isIgnore 是否忽略大小写
     * @param str      待校验的字符串
     * @param strs     待校验的字符串数组
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isEndsWith(final boolean isIgnore, final String str, final String... strs) {
        if (!isEmpty(str) && strs != null && strs.length != 0) {
            String tempString = str;
            // 判断是否需要忽略大小写
            if (isIgnore) {
                tempString = tempString.toLowerCase();
            }
            // 获取数据长度
            int len = strs.length;
            // 遍历判断
            for (int i = 0; i < len; i++) {
                // 获取临时变量
                String val = strs[i];
                // 判断是否为 null, 或者长度为 0
                if (!isEmpty(val)) {
                    if (isIgnore) {
                        // 转换小写
                        String valIgnore = val.toLowerCase();
                        // 判断是否属于 val 结尾
                        if (tempString.endsWith(valIgnore)) {
                            return true;
                        }
                    } else {
                        // 判断是否属于 val 结尾
                        if (tempString.endsWith(val)) {
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    // ============
    // = 其他处理 =
    // ============

    /**
     * 判断字符串是否为 null 或全为空白字符
     * @param str 待校验字符串
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isSpace(final String str) {
        if (str == null) return true;
        for (int i = 0, len = str.length(); i < len; ++i) {
            if (!Character.isWhitespace(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    /**
     * 清空字符串全部空格
     * @param str 待处理字符串
     * @return 处理后的字符串
     */
    public static String toClearSpace(final String str) {
        if (isEmpty(str)) return str;
        return str.replaceAll(" ", "");
    }

    /**
     * 清空字符串前后所有空格
     * @param str 待处理字符串
     * @return 处理后的字符串
     */
    public static String toClearSpaceTrim(final String str) {
        if (isEmpty(str)) return str;
        String tempString = str;
        // 如果前面或者后面都是空格开头, 就一直进行处理
        while (tempString.startsWith(" ") || tempString.endsWith(" ")) {
            tempString = tempString.trim();
        }
        return tempString;
    }

    // =

    /**
     * 追加空格
     * @param number 空格数量
     * @return 指定数量的空格字符串
     */
    public static String appendSpace(final int number) {
        StringBuilder builder = new StringBuilder();
        if (number > 0) {
            for (int i = 0; i < number; i++) {
                builder.append(" ");
            }
        }
        return builder.toString();
    }

    /**
     * 追加 Tab
     * @param number tab 键数量
     * @return 指定数量的 Tab 字符串
     */
    public static String appendTab(final int number) {
        StringBuilder builder = new StringBuilder();
        if (number > 0) {
            for (int i = 0; i < number; i++) {
                builder.append("\t");
            }
        }
        return builder.toString();
    }

    /**
     * 追加换行
     * @param number 换行数量
     * @return 指定数量的换行字符串
     */
    public static String appendLine(final int number) {
        StringBuilder builder = new StringBuilder();
        if (number > 0) {
            for (int i = 0; i < number; i++) {
                builder.append(NEW_LINE_STR);
            }
        }
        return builder.toString();
    }

    /**
     * 冒号分割处理
     * @param str 待处理字符串
     * @return 冒号分割后的字符串
     */
    public static String colonSplit(final String str) {
        if (!isEmpty(str)) {
            return str.replaceAll("(?<=[0-9A-F]{2})[0-9A-F]{2}", ":$0");
        }
        return str;
    }

    // =

    /**
     * 检查字符串
     * @param str 待校验字符串
     * @return 如果待校验字符串为 null, 则返回默认字符串, 如果不为 null, 则返回该字符串
     */
    public static String toCheckValue(final String str) {
        return toCheckValue("", str);
    }

    /**
     * 检查字符串
     * @param defaultStr 默认字符串
     * @param str        待校验字符串
     * @return 如果待校验字符串为 null, 则返回 defaultStr, 如果不为 null, 则返回该字符串
     */
    public static String toCheckValue(final String defaultStr, final String str) {
        return isEmpty(str) ? defaultStr : str;
    }

    /**
     * 检查字符串 - 单独检查两个值
     * @param defaultStr 默认字符串
     * @param value1     第一个待校验字符串
     * @param value2     第二个待校验字符串
     * @return 两个待校验字符串中不为 null 的字符串, 如果都为 null, 则返回 defaultStr
     */
    public static String toCheckValue(final String defaultStr, final String value1, final String value2) {
        if (isEmpty(value1)) {
            if (isEmpty(value2)) {
                return defaultStr;
            } else {
                return value2;
            }
        } else {
            return value1;
        }
    }

    /**
     * 检查字符串 - 多个值
     * @param defaultStr 默认字符串
     * @param strs       待校验字符串数组
     * @return 字符串数组中不为 null 的字符串, 如果都为 null, 则返回 defaultStr
     */
    public static String toCheckValues(final String defaultStr, final String... strs) {
        if (strs != null && strs.length != 0) {
            for (int i = 0, len = strs.length; i < len; i++) {
                String val = strs[i];
                if (!isEmpty(val)) {
                    return val;
                }
            }
        }
        return defaultStr;
    }

    /**
     * 检查字符串 - 多个值 ( 删除前后空格对比判断 )
     * @param defaultStr 默认字符串
     * @param strs       待校验字符串数组
     * @return 字符串数组中不为 null 的字符串, 如果都为 null, 则返回 defaultStr
     */
    public static String toCheckValuesSpace(final String defaultStr, final String... strs) {
        if (strs != null && strs.length != 0) {
            for (int i = 0, len = strs.length; i < len; i++) {
                // 删除前后空格处理后, 进行返回
                String val = toClearSpaceTrim(strs[i]);
                if (!isEmpty(val)) {
                    return val;
                }
            }
        }
        return defaultStr;
    }

    // ==================
    // = 数据格式化处理 =
    // ==================

    /**
     * 获取格式化后的字符串
     * @param format 待格式化字符串
     * @param args   格式化参数
     * @return 格式化后的字符串
     */
    public static String getFormatString(final String format, final Object... args) {
        if (format == null) return null;
        try {
            if (args != null && args.length != 0) {
                return String.format(format, args);
            } else {
                return format;
            }
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "getFormatString");
        }
        return null;
    }

    // =

    /**
     * 获取自动数量格式化后的字符串 ( 可变参数 )
     * @param args 格式化参数
     * @return 格式化后的字符串
     */
    public static String getAutoFormatString(final Object... args) {
        if (args != null && args.length != 0) {
            try {
                int length = args.length;
                StringBuilder builder = new StringBuilder();
                builder.append("%s");
                if (length > 1) {
                    for (int i = 1; i < length; i++) {
                        builder.append(" %s");
                    }
                }
                return String.format(builder.toString(), args);
            } catch (Exception e) {
                JCLogUtils.eTag(TAG, e, "getAutoFormatString");
            }
        }
        return null;
    }

    /**
     * 获取自动数量格式化后的字符串 ( 可变参数 )
     * @param args 格式化参数
     * @return 格式化后的字符串
     */
    public static String getAutoFormatString2(final Object... args) {
        if (args != null && args.length != 0) {
            try {
                int length = args.length;
                StringBuilder builder = new StringBuilder();
                builder.append("[%s]");
                if (length > 1) {
                    for (int i = 1; i < length; i++) {
                        builder.append(" %s");
                    }
                }
                return String.format(builder.toString(), args);
            } catch (Exception e) {
                JCLogUtils.eTag(TAG, e, "getAutoFormatString2");
            }
        }
        return null;
    }

    // =

    /**
     * StringBuilder 拼接处理
     * @param args 拼接数据源
     * @return {@link StringBuilder}
     */
    public static StringBuilder appends(final Object... args) {
        return appends(new StringBuilder(), null, false, args);
    }

    /**
     * StringBuilder 拼接处理
     * @param split 追加间隔
     * @param args  拼接数据源
     * @return {@link StringBuilder}
     */
    public static StringBuilder appends(final String split, final Object... args) {
        return appends(new StringBuilder(), split, false, args);
    }

    /**
     * StringBuilder 拼接处理
     * @param builder 拼接 Builder
     * @param split   追加间隔
     * @param args    拼接数据源
     * @return {@link StringBuilder}
     */
    public static StringBuilder appends(final StringBuilder builder, final String split, final Object... args) {
        return appends(builder, split, false, args);
    }

    /**
     * StringBuilder 拼接处理
     * @param builder 拼接 Builder
     * @param split   追加间隔
     * @param end     结尾是否追加
     * @param args    拼接数据源
     * @return {@link StringBuilder}
     */
    public static StringBuilder appends(final StringBuilder builder, final String split, final boolean end, final Object... args) {
        if (builder != null && args != null) {
            // 获取间隔字符串, 优化循环判断
            String str = isEmpty(split) ? "" : split;
            // 循环处理
            for (int i = 0, len = args.length; i < len; i++) {
                builder.append(args[i]); // 拼接数据
                // 判断是否结尾
                if (len - i == 1) {
                    // 判断结尾是否追加
                    if (end) builder.append(str); // 间隔追加
                } else {
                    builder.append(str); // 间隔追加
                }
            }
            return builder;
        }
        return builder;
    }

    // ============
    // = 转换处理 =
    // ============

    /**
     * 字符串进行 GBK 编码
     * @param str 待处理字符串
     * @return GBK 编码后的字符串
     */
    public static String toGBKEncode(final String str) {
        return toStrEncode(str, "GBK");
    }

    /**
     * 字符串进行 GBK2312 编码
     * @param str 待处理字符串
     * @return GBK2312 编码后的字符串
     */
    public static String toGBK2312Encode(final String str) {
        return toStrEncode(str, "GBK-2312");
    }

    /**
     * 字符串进行 UTF-8 编码
     * @param str 待处理字符串
     * @return UTF-8 编码后的字符串
     */
    public static String toUTF8Encode(final String str) {
        return toStrEncode(str, "UTF-8");
    }

    /**
     * 进行字符串编码
     * @param str 待处理字符串
     * @param enc 编码格式
     * @return 指定编码格式编码后的字符串
     */
    public static String toStrEncode(final String str, final String enc) {
        if (str == null || enc == null) return null;
        try {
            return new String(str.getBytes(), enc);
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toStrEncode");
        }
        return str;
    }

    // =

    /**
     * 进行 URL 编码, 默认 UTF-8
     * @param str 待处理字符串
     * @return UTF-8 编码格式 URL 编码后的字符串
     */
    public static String toUrlEncode(final String str) {
        return toUrlEncode(str, "UTF-8");
    }

    /**
     * 进行 URL 编码
     * @param str 待处理字符串
     * @param enc 编码格式
     * @return 指定编码格式 URL 编码后的字符串
     */
    public static String toUrlEncode(final String str, final String enc) {
        if (str == null || enc == null) return null;
        try {
            return URLEncoder.encode(str, enc);
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toUrlEncode");
        }
        return null;
    }

    // =

    /**
     * 进行 URL 解码, 默认 UTF-8
     * @param str 待处理字符串
     * @return UTF-8 编码格式 URL 解码后的字符串
     */
    public static String toUrlDecode(final String str) {
        return toUrlDecode(str, "UTF-8");
    }

    /**
     * 进行 URL 解码
     * @param str 待处理字符串
     * @param enc 解码格式
     * @return 指定编码格式 URL 解码后的字符串
     */
    public static String toUrlDecode(final String str, final String enc) {
        if (str == null || enc == null) return null;
        try {
            return URLDecoder.decode(str, enc);
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toUrlDecode");
        }
        return null;
    }

    // =

    /**
     * 将字符串转移为 ASCII 码
     * @param str 待处理字符串
     * @return 字符串转 ASCII 码后的字符串
     */
    public static String toASCII(final String str) {
        if (isEmpty(str)) return str;
        try {
            StringBuilder builder = new StringBuilder();
            byte[] bytes = str.getBytes();
            for (int i = 0, len = bytes.length; i < len; i++) {
                builder.append(Integer.toHexString(bytes[i] & 0xff));
            }
            return builder.toString();
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toASCII");
        }
        return null;
    }

    /**
     * 将字符串转移为 Unicode 码
     * @param str 待处理字符串
     * @return 字符串转 Unicode 码后的字符串
     */
    public static String toUnicode(final String str) {
        if (isEmpty(str)) return str;
        try {
            StringBuilder builder = new StringBuilder();
            char[] chars = str.toCharArray();
            for (int i = 0, len = chars.length; i < len; i++) {
                builder.append("\\u").append(Integer.toHexString(chars[i]));
            }
            return builder.toString();
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toUnicode");
        }
        return null;
    }

    /**
     * 将字符数组转移为 Unicode 码
     * @param chars char[]
     * @return char[] 转 Unicode 码后的字符串
     */
    public static String toUnicodeString(final char[] chars) {
        if (chars == null) return null;
        try {
            StringBuilder builder = new StringBuilder();
            for (int i = 0, len = chars.length; i < len; i++) {
                builder.append("\\u").append(Integer.toHexString(chars[i]));
            }
            return builder.toString();
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toUnicodeString");
        }
        return null;
    }

    /**
     * 转化为半角字符
     * @param str 待处理字符串
     * @return 转换半角字符串
     */
    public static String toDBC(final String str) {
        if (isEmpty(str)) return str;
        char[] chars = str.toCharArray();
        for (int i = 0, len = chars.length; i < len; i++) {
            if (chars[i] == 12288) {
                chars[i] = ' ';
            } else if (65281 <= chars[i] && chars[i] <= 65374) {
                chars[i] = (char) (chars[i] - 65248);
            } else {
                chars[i] = chars[i];
            }
        }
        return new String(chars);
    }

    /**
     * 转化为全角字符 如: a = ａ, A = Ａ
     * @param str 待处理字符串
     * @return 转换全角字符串
     */
    public static String toSBC(final String str) {
        if (isEmpty(str)) return str;
        char[] chars = str.toCharArray();
        for (int i = 0, len = chars.length; i < len; i++) {
            if (chars[i] == ' ') {
                chars[i] = (char) 12288;
            } else if (33 <= chars[i] && chars[i] <= 126) {
                chars[i] = (char) (chars[i] + 65248);
            } else {
                chars[i] = chars[i];
            }
        }
        return new String(chars);
    }

    // =

    /**
     * 检测字符串是否全是中文
     * @param str 待校验字符串
     * @return {@code true} yes, {@code false} no
     */
    public static boolean checkChineseToString(final String str) {
        if (isEmpty(str)) return false;
        boolean result = true;
        char[] chars = str.toCharArray();
        for (int i = 0, len = chars.length; i < len; i++) {
            if (!isChinese(chars[i])) {
                result = false;
                break;
            }
        }
        return result;
    }

    /**
     * 判定输入汉字
     * @param ch 待校验字符
     * @return {@code true} yes, {@code false} no
     */
    public static boolean isChinese(final char ch) {
        Character.UnicodeBlock ub = Character.UnicodeBlock.of(ch);
        return ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS
                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A
                || ub == Character.UnicodeBlock.GENERAL_PUNCTUATION
                || ub == Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION
                || ub == Character.UnicodeBlock.HALFWIDTH_AND_FULLWIDTH_FORMS;
    }

    // ==================
    // = 字符串处理方法 =
    // ==================

    /**
     * 首字母大写
     * @param str 待处理字符串
     * @return 首字母大写字符串
     */
    public static String upperFirstLetter(final String str) {
        if (isEmpty(str) || !Character.isLowerCase(str.charAt(0))) return str;
        try {
            return (char) (str.charAt(0) - 32) + str.substring(1);
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "upperFirstLetter");
            return str;
        }
    }

    /**
     * 首字母小写
     * @param str 待处理字符串
     * @return 首字母小写字符串
     */
    public static String lowerFirstLetter(final String str) {
        if (isEmpty(str) || !Character.isUpperCase(str.charAt(0))) return str;
        try {
            return (char) (str.charAt(0) + 32) + str.substring(1);
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "lowerFirstLetter");
            return str;
        }
    }

    /**
     * 反转字符串
     * @param str 待处理字符串
     * @return 反转字符串
     */
    public static String reverse(final String str) {
        int len = length(str);
        if (len <= 1) return str;
        int mid = len >> 1;
        char[] chars = str.toCharArray();
        char ch;
        for (int i = 0; i < mid; ++i) {
            ch = chars[i];
            chars[i] = chars[len - i - 1];
            chars[len - i - 1] = ch;
        }
        return new String(chars);
    }

    /**
     * 字符串连接, 将参数列表拼接为一个字符串
     * @param args 追加数据
     * @return 拼接后的字符串
     */
    public static String concat(final Object... args) {
        return concatSpiltWith("", args);
    }

    /**
     * 字符串连接, 将参数列表拼接为一个字符串
     * @param startStr 开始字符串
     * @param args     追加数据
     * @return 拼接后的字符串
     */
    public static String concatSpiltWith(final String startStr, final Object... args) {
        if (args == null) return null;
        StringBuilder builder = new StringBuilder();
        for (int i = 0, len = args.length; i < len; i++) {
            if (i != 0) builder.append(startStr);
            builder.append(args[i]);
        }
        return builder.toString();
    }

    /**
     * 下划线命名转为驼峰命名
     * @param str 下划线命名格式字符串
     * @return 驼峰命名格式字符串
     */
    public static String underScoreCaseToCamelCase(final String str) {
        if (isEmpty(str)) return str;
        if (!str.contains("_")) return str;
        StringBuilder builder = new StringBuilder();
        char[] chars = str.toCharArray();
        boolean hitUnderScore = false;
        builder.append(chars[0]);
        for (int i = 1, len = chars.length; i < len; i++) {
            char c = chars[i];
            if (c == '_') {
                hitUnderScore = true;
            } else {
                if (hitUnderScore) {
                    builder.append(Character.toUpperCase(c));
                    hitUnderScore = false;
                } else {
                    builder.append(c);
                }
            }
        }
        return builder.toString();
    }

    /**
     * 驼峰命名法转为下划线命名
     * @param str 驼峰命名格式字符串
     * @return 下划线命名格式字符串
     */
    public static String camelCaseToUnderScoreCase(final String str) {
        if (isEmpty(str)) return str;
        StringBuilder builder = new StringBuilder();
        char[] chars = str.toCharArray();
        for (int i = 0, len = chars.length; i < len; i++) {
            char c = chars[i];
            if (Character.isUpperCase(c)) {
                builder.append("_").append(Character.toLowerCase(c));
            } else {
                builder.append(c);
            }
        }
        return builder.toString();
    }

    /**
     * 字符串数据库字符转义
     * @param str 待处理字符串
     * @return 转义处理后的字符串
     */
    public static String sqliteEscape(final String str) {
        if (isEmpty(str)) return str;
        String keyWord = str;
        // 替换关键字
        keyWord = keyWord.replace("/", "//");
        keyWord = keyWord.replace("'", "''");
        keyWord = keyWord.replace("[", "/[");
        keyWord = keyWord.replace("]", "/]");
        keyWord = keyWord.replace("%", "/%");
        keyWord = keyWord.replace("&", "/&");
        keyWord = keyWord.replace("_", "/_");
        keyWord = keyWord.replace("(", "/(");
        keyWord = keyWord.replace(")", "/)");
        return keyWord;
    }

    // ==============
    // = 字符串处理 =
    // ==============

    /**
     * 转换手机号
     * @param phone 待处理字符串
     * @return 处理后的字符串
     */
    public static String converHideMobile(final String phone) {
        return converHideMobile(phone, "*");
    }

    /**
     * 转换手机号
     * @param phone  待处理字符串
     * @param symbol 转换符号
     * @return 处理后的字符串
     */
    public static String converHideMobile(final String phone, final String symbol) {
        return converSymbolHide(3, phone, symbol);
    }

    /**
     * 转换符号处理
     * @param start  开始位置
     * @param str    待处理字符串
     * @param symbol 转换符号
     * @return 处理后的字符串
     */
    public static String converSymbolHide(final int start, final String str, final String symbol) {
        if (!isEmpty(str)) {
            if (start <= 0) {
                return str;
            }
            // 获取数据长度
            int length = str.length();
            // 如果数据小于 start 位则直接返回
            if (length <= start) {
                return str;
            } else { // 大于 start 位
                StringBuilder builder = new StringBuilder();
                builder.append(str.substring(0, start));
                int len = length - start;
                // 进行平分
                len /= 2;
                // 进行遍历保存
                for (int i = 0; i < len; i++) {
                    builder.append(symbol);
                }
                builder.append(str.substring(start + len, length));
                return builder.toString();
            }
        }
        return "";
    }

    // =

    /**
     * 裁剪超出的内容, 并且追加符号 ( 如 ...)
     * @param maxLength 允许最大的长度
     * @param str       待处理字符串
     * @param symbol    转换符号
     * @return 处理后的字符串
     */
    public static String subEllipsize(final int maxLength, final String str, final String symbol) {
        if (maxLength >= 1) {
            // 获取内容长度
            int strLength = length(str);
            // 防止为不存在数据
            if (strLength != 0) {
                if (maxLength >= strLength) {
                    return str;
                }
                return str.substring(0, maxLength) + toCheckValue(symbol);
            }
        }
        return "";
    }

    /**
     * 裁剪符号处理
     * @param start        开始位置
     * @param symbolNumber 转换数量
     * @param str          待处理字符串
     * @param symbol       转换符号
     * @return 处理后的字符串
     */
    public static String subSymbolHide(final int start, final int symbolNumber, final String str, final String symbol) {
        if (!isEmpty(str)) {
            if (start <= 0 || symbolNumber <= 0) {
                return str;
            }
            // 获取数据长度
            int length = str.length();
            // 如果数据小于 start 位则直接返回
            if (length <= start) {
                return str;
            } else { // 大于 start 位
                StringBuilder builder = new StringBuilder();
                builder.append(str.substring(0, start));
                int len = length - start - symbolNumber;
                // 如果超出总长度, 则进行控制
                if (len <= 0) { // 表示后面的全部转换
                    len = length - start;
                } else { // 需要裁剪的数量
                    len = symbolNumber;
                }
                // 进行遍历保存
                for (int i = 0; i < len; i++) {
                    builder.append(symbol);
                }
                builder.append(str.substring(start + len, length));
                return builder.toString();
            }
        }
        return "";
    }

    /**
     * 裁剪内容, 设置符号处理
     * @param str               待处理字符串
     * @param frontRetainLength 前面保留的长度
     * @param rearRetainLength  后面保留的长度
     * @param symbol            转换符号
     * @return 处理后的字符串
     */
    public static String subSetSymbol(final String str, final int frontRetainLength, final int rearRetainLength, final String symbol) {
        if (str != null) {
            try {
                // 截取前面需保留的内容
                String startStr = str.substring(0, frontRetainLength);
                // 截取后面需保留的内容
                String endStr = str.substring(str.length() - rearRetainLength);
                // 特殊符号长度
                int symbolLength = str.length() - (frontRetainLength + rearRetainLength);
                if (symbolLength >= 1) {
                    StringBuilder builder = new StringBuilder();
                    for (int i = 0; i < symbolLength; i++) {
                        builder.append(symbol);
                    }
                    return startStr + builder.toString() + endStr;
                }
                return startStr + endStr;
            } catch (Exception e) {
                JCLogUtils.eTag(TAG, e, "subSetSymbol");
            }
        }
        return null;
    }

    // ==================
    // = 替换、截取操作 =
    // ==================

    /**
     * 裁剪字符串
     * @param str      需要裁剪的字符串
     * @param endIndex 结束裁剪的位置
     * @return 裁剪后的字符串
     */
    public static String substring(final String str, final int endIndex) {
        return substring(str, 0, endIndex, true);
    }

    /**
     * 裁剪字符串
     * @param str      需要裁剪的字符串
     * @param endIndex 结束裁剪的位置
     * @param isReturn 开始位置超过限制是否返回内容
     * @return 裁剪后的字符串
     */
    public static String substring(final String str, final int endIndex, final boolean isReturn) {
        return substring(str, 0, endIndex, isReturn);
    }

    /**
     * 裁剪字符串
     * @param str        需要裁剪的字符串
     * @param beginIndex 开始裁剪的位置
     * @param endIndex   结束裁剪的位置
     * @param isReturn   开始位置超过限制是否返回内容
     * @return 裁剪后的字符串
     */
    public static String substring(final String str, final int beginIndex, final int endIndex, final boolean isReturn) {
        if (!isEmpty(str) && beginIndex >= 0 && endIndex >= 0 && endIndex >= beginIndex) {
            // 获取数据长度
            int len = length(str);
            // 防止超过限制
            if (beginIndex > len) {
                return isReturn ? str : "";
            }
            // 防止超过限制
            return str.substring(beginIndex, (endIndex >= len) ? len : endIndex);
        }
        return isReturn ? str : "";
    }

    // =

    /**
     * 替换 ( 删除 - 替换成 "") 字符串中符合 特定标记字符的 startsWith - endsWith
     * * 如 _____a_a_a_a________ 传入 _ 等于 ____a_a_a_a____
     * @param str    待处理字符串
     * @param suffix 替换符号字符串
     * @return 处理后的字符串
     */
    public static String toReplaceSEWith(final String str, final String suffix) {
        return toReplaceSEWith(str, suffix, "");
    }

    /**
     * 替换字符串中符合 特定标记字符的 startsWith - endsWith
     * 如 _____a_a_a_a________ 传入 _, c 等于 c____a_a_a_a____c
     * @param str    待处理字符串
     * @param suffix 替换匹配内容
     * @param value  需要替换的内容
     * @return 处理后的字符串
     */
    public static String toReplaceSEWith(final String str, final String suffix, final String value) {
        try {
            if (isEmpty(str) || isEmpty(suffix) || isEmpty(value) || suffix.equals(value))
                return str;
            // 获取编辑内容长度
            int suffixLength = suffix.length();
            // 保存新的 Builder 中, 减少内存开销
            StringBuilder builder = new StringBuilder(str);
            // 判断是否在最头部
            if (builder.indexOf(suffix) == 0) {
                builder.delete(0, suffixLength);
                // 追加内容
                builder.insert(0, value);
            }
            // 获取尾部的位置
            int lastIndexOf = -1;
            // 数据长度
            int bufLength = -1;
            // 判断是否在最尾部
            if ((lastIndexOf = builder.lastIndexOf(suffix)) == ((bufLength = builder.length()) - suffixLength)) {
                builder.delete(lastIndexOf, bufLength);
                // 追加内容
                builder.insert(lastIndexOf, value);
            }
            return builder.toString();
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toReplaceSEWith");
        }
        return str;
    }

    // =

    /**
     * 替换开头字符串
     * @param str    待处理字符串
     * @param prefix 开头匹配字符串
     * @return 处理后的字符串
     */
    public static String toReplaceStartsWith(final String str, final String prefix) {
        return toReplaceStartsWith(str, prefix, "");
    }

    /**
     * 替换开头字符串
     * @param str         待处理字符串
     * @param prefix      开头匹配字符串
     * @param startAppend 开头追加的内容
     * @return 处理后的字符串
     */
    public static String toReplaceStartsWith(final String str, final String prefix, final String startAppend) {
        if (!isEmpty(str) && !isEmpty(prefix)) {
            try {
                if (str.startsWith(prefix)) {
                    return toCheckValue(startAppend) + str.substring(prefix.length());
                }
            } catch (Exception e) {
                JCLogUtils.eTag(TAG, e, "toReplaceStartsWith");
            }
        }
        return str;
    }

    /**
     * 替换结尾字符串
     * @param str    待处理字符串
     * @param suffix 结尾匹配字符串
     * @return 处理后的字符串
     */
    public static String toReplaceEndsWith(final String str, final String suffix) {
        return toReplaceEndsWith(str, suffix, "");
    }

    /**
     * 替换结尾字符串
     * @param str    待处理字符串
     * @param suffix 结尾匹配字符串
     * @param value  需要替换的内容
     * @return 处理后的字符串
     */
    public static String toReplaceEndsWith(final String str, final String suffix, final String value) {
        if (!isEmpty(str) && !isEmpty(suffix)) {
            try {
                if (str.endsWith(suffix)) {
                    return str.substring(0, str.length() - suffix.length()) + value;
                }
            } catch (Exception e) {
                JCLogUtils.eTag(TAG, e, "toReplaceEndsWith");
            }
        }
        return str;
    }

    // =

    /**
     * 这个方法功能主要把字符符合标记的 头部和尾部都替换成 ""
     * 如 _____a_a_a_a________ 传入 _ 等于 a_a_a_a
     * 替换字符串中符合 特定标记字符的 startsWith(indexOf) - endsWith(lastIndexOf)
     * @param str    待处理字符串
     * @param suffix 匹配判断字符串
     * @return 处理后的字符串
     */
    public static String toClearSEWiths(final String str, final String suffix) {
        if (isEmpty(str) || isEmpty(suffix)) return str;
        try {
            // 获取编辑内容长度
            int suffixLength = suffix.length();
            // 保存新的 Builder 中, 减少内存开销
            StringBuilder builder = new StringBuilder(str);
            // 进行循环判断 - 属于最前面的, 才进行处理
            while (builder.indexOf(suffix) == 0) {
                builder.delete(0, suffixLength);
            }
            // 获取尾部的位置
            int lastIndexOf = -1;
            // 数据长度
            int bufLength = -1;
            // 进行循环判断 - 属于最后面的, 才进行处理
            while ((lastIndexOf = builder.lastIndexOf(suffix)) == ((bufLength = builder.length()) - suffixLength)) {
                builder.delete(lastIndexOf, bufLength);
            }
            return builder.toString();
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toClearSEWiths");
        }
        return str;
    }

    /**
     * 清空属于特定字符串开头的字段
     * 如 _____a_a_a_a________ 传入 _ 等于 a_a_a_a_____
     * 替换字符串中符合 特定标记字符的 endsWith(lastIndexOf)
     * @param str    待处理字符串
     * @param suffix 匹配判断字符串
     * @return 处理后的字符串
     */
    public static String toClearStartsWith(final String str, final String suffix) {
        if (isEmpty(str) || isEmpty(suffix)) return str;
        try {
            // 获取编辑内容长度
            int suffixLength = suffix.length();
            // 保存新的 Builder 中, 减少内存开销
            StringBuilder builder = new StringBuilder(str);
            // 进行循环判断 - 属于最前面的, 才进行处理
            while (builder.indexOf(suffix) == 0) {
                builder.delete(0, suffixLength);
            }
            return builder.toString();
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toClearStartsWith");
        }
        return str;
    }

    /**
     * 清空属于特定字符串结尾的字段
     * 如 _____a_a_a_a________ 传入 _ 等于 _____a_a_a_a
     * 替换字符串中符合 特定标记字符的 endsWith(lastIndexOf)
     * @param str    待处理字符串
     * @param suffix 匹配判断字符串
     * @return 处理后的字符串
     */
    public static String toClearEndsWith(final String str, final String suffix) {
        if (isEmpty(str) || isEmpty(suffix)) return str;
        try {
            // 获取编辑内容长度
            int suffixLength = suffix.length();
            // 保存新的 Builder 中, 减少内存开销
            StringBuilder builder = new StringBuilder(str);
            // 获取最后一位位置
            int bufLength = 0;
            // 进行循环判断 - 属于最前面的, 才进行处理
            while (builder.lastIndexOf(suffix) == ((bufLength = builder.length()) - suffixLength)) {
                builder.delete(bufLength - suffixLength, bufLength);
            }
            return builder.toString();
        } catch (Exception e) {
            JCLogUtils.eTag(TAG, e, "toClearEndsWith");
        }
        return str;
    }

    // =

    /**
     * 替换字符串
     * @param str     待处理字符串
     * @param suffix  匹配判断字符串
     * @param replace 替换的内容
     * @return 处理后的字符串
     */
    public static String replaceStr(final String str, final String suffix, final String replace) {
        // 如果替换的内容或者判断的字符串为 null, 则直接跳过
        if (!isEmpty(str) && !isEmpty(suffix) && replace != null && !suffix.equals(replace)) {
            try {
                return str.replaceAll(suffix, replace);
            } catch (Exception e) {
                JCLogUtils.eTag(TAG, e, "replaceStr");
            }
        }
        return str;
    }

    /**
     * 替换字符串
     * @param str     待处理字符串
     * @param suffix  匹配判断字符串
     * @param replace 替换的内容
     * @return 处理后的字符串, 替换失败则返回 null
     */
    public static String replaceStrToNull(final String str, final String suffix, final String replace) {
        // 如果替换的内容或者判断的字符串为 null, 则直接跳过
        if (!isEmpty(str) && !isEmpty(suffix) && replace != null && !suffix.equals(replace)) {
            try {
                return str.replaceAll(suffix, replace);
            } catch (Exception e) {
                JCLogUtils.eTag(TAG, e, "replaceStrToNull");
            }
        }
        return null;
    }

    /**
     * 替换字符串
     * @param str         内容
     * @param suffixArys  匹配判断字符串数组
     * @param replaceArys 准备替换的字符串数组
     * @return 处理后的字符串
     */
    public static String replaceStrs(final String str, final String[] suffixArys, final String[] replaceArys) {
        // 防止数据为 null
        if (str != null && suffixArys != null && replaceArys != null) {
            String tempString = str;
            // 替换的特殊字符串长度
            int spCount = suffixArys.length;
            // 替换的内容长度
            int reCount = replaceArys.length;
            // 相同才进行处理
            if (spCount == reCount) {
                // 遍历进行判断
                for (int i = 0; i < spCount; i++) {
                    // 进行替换字符串
                    tempString = replaceStr(tempString, suffixArys[i], replaceArys[i]);
                }
                return tempString;
            }
        }
        return null;
    }
}